# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from collective.geo.faceted.testing import COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING  # noqa
from plone import api

import unittest


class TestSetup(unittest.TestCase):
    """Test that collective.geo.faceted is properly installed."""

    layer = COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if collective.geo.faceted is installed."""
        self.assertTrue(self.installer.isProductInstalled(
            'collective.geo.faceted'))

    def test_browserlayer(self):
        """Test that ICollectiveGeoFacetedLayer is registered."""
        from collective.geo.faceted.interfaces import (
            ICollectiveGeoFacetedLayer)
        from plone.browserlayer import utils
        self.assertIn(ICollectiveGeoFacetedLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.uninstallProducts(['collective.geo.faceted'])

    def test_product_uninstalled(self):
        """Test if collective.geo.faceted is cleanly uninstalled."""
        self.assertFalse(self.installer.isProductInstalled(
            'collective.geo.faceted'))

    def test_browserlayer_removed(self):
        """Test that ICollectiveGeoFacetedLayer is removed."""
        from collective.geo.faceted.interfaces import ICollectiveGeoFacetedLayer  # noqa
        from plone.browserlayer import utils
        self.assertNotIn(ICollectiveGeoFacetedLayer, utils.registered_layers())



# -*- coding: utf-8 -*-
from collective.geo.json.browser.jsonview import JsonFolderDocument


class MapView(JsonFolderDocument):

    def json(self):
        view = self.context.restrictedTraverse('faceted_query')
        brains = view.query(batch=False)
        self.set_brains(brains)
        json = self.get_json()
        return json

    def set_brains(self, brains):
        self.brains = brains

    def get_brains(self):
        """Needed by get_json method from collective.geo.json."""
        return self.brains



# -*- coding: utf-8 -*-
""" Adapters
"""
from eea.facetednavigation.interfaces import IWidgetFilterBrains
from zope.interface import implementer

import Missing


@implementer(IWidgetFilterBrains)
class WidgetFilterBrains(object):
    """ Filter brains after query
    """

    def __init__(self, context):
        self.widget = context

    def __call__(self, brains, form):
        """ Filter brains
        """
        if set(['west', 'east', 'north', 'south']).issubset(form.keys()):
            north = float(form.get('north'))
            east = float(form.get('east'))
            south = float(form.get('south'))
            west = float(form.get('west'))
            for brain in brains:
                if brain.zgeo_geometry is not Missing.Value:
                    zgeo_geometry = brain.zgeo_geometry
                    geo_type = brain.zgeo_geometry.get('type', None)
                    if geo_type == 'Point':
                        lon, lat = zgeo_geometry['coordinates']
                        if south <= lat <= north and west <= lon <= east:
                            yield brain
                    elif geo_type in ['Polygon', 'Line']:
                        in_map = False
                        for lon, lat in zgeo_geometry['coordinates'][0]:
                            # see polygon if one of is point is in map
                            if south <= lat <= north and west <= lon <= east:
                                in_map = True
                        if in_map:
                            yield brain



# -*- coding: utf-8 -*-
from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import collective.geo.faceted


class CollectiveGeoFacetedLayer(PloneSandboxLayer):

    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        self.loadZCML(package=collective.geo.faceted, name='testing.zcml')

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'collective.geo.faceted:testing')


COLLECTIVE_GEO_FACETED_FIXTURE = CollectiveGeoFacetedLayer()


COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING = IntegrationTesting(
    bases=(COLLECTIVE_GEO_FACETED_FIXTURE,),
    name='CollectiveGeoFacetedLayer:IntegrationTesting'
)


COLLECTIVE_GEO_FACETED_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(COLLECTIVE_GEO_FACETED_FIXTURE,),
    name='CollectiveGeoFacetedLayer:FunctionalTesting'
)


COLLECTIVE_GEO_FACETED_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        COLLECTIVE_GEO_FACETED_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE
    ),
    name='CollectiveGeoFacetedLayer:AcceptanceTesting'
)



# -*- coding: utf-8 -*-
from collective.geo.leaflet.interfaces import IGeoMap
from eea.facetednavigation.config import ANNO_FACETED_LAYOUT
from plone.app.layout.viewlets import common
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.annotation.interfaces import IAnnotations


class MapViewlet(common.ViewletBase):

    index = ViewPageTemplateFile('mapviewlet.pt')

    def avaiable(self):
        if not getattr(self.context, 'layout', '') == 'facetednavigation_view':
            return False
        view_name = IAnnotations(self.context).get(
            ANNO_FACETED_LAYOUT, 'faceted-preview-items')
        if not view_name == 'faceted-map-view':
            return False
        return True

    @property
    def geomap(self):
        return IGeoMap(self.context)



# -*- coding: utf-8 -*-
from collective.geo.faceted.testing import COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING  # noqa
from eea.facetednavigation.interfaces import IFacetedNavigable
from plone import api
from zope.interface import directlyProvides

import unittest


class TestView(unittest.TestCase):

    layer = COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def test_map_view(self):
        coll = api.content.create(
            type='Collection', container=self.portal, id='collection')
        view = coll.restrictedTraverse('faceted-single-map-view')
        directlyProvides(coll, IFacetedNavigable)
        json = view.json()
        self.assertEqual(
            json, '{"type": "FeatureCollection", "features": [], "title": ""}')
        self.assertEqual(len(view.get_brains()), 7)



# -*- coding: utf-8 -*-
from collective.geo.faceted import _
from collective.geo.faceted.widget.interfaces import IBoundsWidget
from eea.facetednavigation.widgets.widget import Widget as AbstractWidget
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from zope.interface import implementer

import logging


logger = logging.getLogger('collective.geo.faceted.')


@implementer(IBoundsWidget)
class Widget(AbstractWidget):
    """ Widget
    """
    # Widget properties
    widget_type = 'bounds'
    widget_label = _('Bounds selection')
    view_js = '++resource++collective.geo.faceted.widget.view.js'
    edit_js = '++resource++collective.geo.faceted.widget.edit.js'
    view_css = '++resource++collective.geo.faceted.widget.view.css'
    # edit_css = '++resource++collective.geo.faceted.widget.edit.css'

    index = ZopeTwoPageTemplateFile('bounds.pt', globals())
    edit_schema = AbstractWidget.edit_schema.copy()
    edit_schema['title'].default = _('Boundary Bounds')

    def __init__(self, context, request, data=None):
        super(Widget, self).__init__(context, request, data)



# -*- coding: utf-8 -*-
from collective.geo.faceted.testing import COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING  # noqa
from collective.geo.faceted.widget.adapters import WidgetFilterBrains
from collective.geo.faceted.widget.bounds import Widget
from collective.geo.geographer.interfaces import IWriteGeoreferenced
from eea.facetednavigation.widgets.storage import Criterion
from plone import api
from zope.component import getMultiAdapter

import unittest


class TestWidget(unittest.TestCase):

    layer = COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.folder = api.content.create(
            container=self.portal,
            id='folder',
            type='Folder'
        )
        subtyper = getMultiAdapter(
            (self.folder, self.request), name=u'faceted_subtyper'
        )
        subtyper.enable()

    def test_bounds_widget_template(self):
        data = Criterion()
        widget = Widget(self.folder, self.request, data=data)
        self.assertIn('style="display: none"', widget.index())
        self.assertIn('faceted-bounds-widget', widget.index())

    def test_bounds_widget_adapter(self):
        # data = Criterion()
        form = {
            'north': '51.00000',
            'east': '6.000',
            'south': '49.000',
            'west': '5.000'
        }
        widget = WidgetFilterBrains(self.folder)
        # document and event have ICoordinates behavior
        event = api.content.create(
            container=self.portal,
            id='event',
            type='Event'
        )
        doc = api.content.create(
            container=self.portal,
            id='doc',
            type='Document'
        )
        geo_doc = IWriteGeoreferenced(doc)
        geo_event = IWriteGeoreferenced(event)
        geo_doc.setGeoInterface('Point', (5.583, 50.633))
        geo_event.setGeoInterface('Point', (5.73, 50.933))
        doc.reindexObject(idxs=['zgeo_geometry', 'collective_geo_styles'])
        event.reindexObject(idxs=['zgeo_geometry', 'collective_geo_styles'])
        # import ipdb; ipdb.set_trace()
        portal_catalog = api.portal.get_tool('portal_catalog')
        query = {}
        query['portal_type'] = ['Event', 'Document']
        brains = portal_catalog(query)
        gen = widget(brains, form)
        geo_brains = [g for g in gen]
        self.assertEqual(len(geo_brains), 2)

        geo_event.setGeoInterface('Point', (6.73, 50.933))
        event.reindexObject(idxs=['zgeo_geometry', 'collective_geo_styles'])
        brains = portal_catalog(query)
        gen = widget(brains, form)
        geo_brains = [g for g in gen]
        self.assertEqual(len(geo_brains), 1)

        geo_doc.setGeoInterface('Point', (5.583, 52.633))
        doc.reindexObject(idxs=['zgeo_geometry', 'collective_geo_styles'])
        brains = portal_catalog(query)
        gen = widget(brains, form)
        geo_brains = [g for g in gen]
        self.assertEqual(len(geo_brains), 0)



# -*- coding: utf-8 -*-
from collective.geo.faceted.testing import COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING  # noqa
from collective.geo.leaflet import geomap
from eea.facetednavigation.config import ANNO_FACETED_LAYOUT
from plone import api
from Products.Five.browser import BrowserView
from zope.annotation.interfaces import IAnnotations
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter
from zope.viewlet.interfaces import IViewletManager

import unittest


class TestViewlet(unittest.TestCase):
    """Test collective.geo.faceted map viewlet"""

    layer = COLLECTIVE_GEO_FACETED_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def get_viewlet(self, context, manager_name, viewlet_name):
        # getting viewlet
        view = BrowserView(self.portal, self.request)
        manager = queryMultiAdapter(
            (context, self.request, view),
            IViewletManager,
            manager_name,
            default=None)
        self.assertIsNotNone(manager)
        manager.update()

        my_viewlet = [
            v for v in manager.viewlets if v.__name__ == viewlet_name]
        self.assertEqual(len(my_viewlet), 1)
        viewlet = my_viewlet[0]
        return viewlet

    def test_avaiable(self):
        context = self.portal
        viewlet = self.get_viewlet(
            context, 'plone.belowcontentbody', 'mapviewlet')
        self.assertFalse(viewlet.avaiable())

        coll = api.content.create(
            type='Collection', container=self.portal, id='collection')
        viewlet = self.get_viewlet(
            coll, 'plone.belowcontentbody', 'mapviewlet')
        subtyper = getMultiAdapter(
            (coll, self.request), name=u'faceted_subtyper')
        subtyper.enable()
        self.assertFalse(viewlet.avaiable())

        annotations = IAnnotations(coll)
        annotations.setdefault(ANNO_FACETED_LAYOUT, 'faceted-map-view')
        self.assertTrue(viewlet.avaiable())

    def test_geomap(self):
        context = self.portal
        viewlet = self.get_viewlet(
            context, 'plone.belowcontentbody', 'mapviewlet')
        self.assertTrue(isinstance(viewlet.geomap, geomap.GeoMap))
